@@ -63,6 +63,7 @@ use core::{
 	marker::{
 		PhantomData,
 		Send,
+		Sync,
 	},
 	mem,
 	ops::{
@@ -299,7 +300,7 @@ where C: Cursor, T: Bits {
 	/// ```
 	pub fn into_boxed_slice(self) -> Box<[T]> {
 		let (ptr, len, _, _) = self.bitptr().raw_parts();
-		let out = unsafe { Vec::from_raw_parts(ptr as *mut T, len, len) }
+		let out = unsafe { Vec::from_raw_parts(ptr.w(), len, len) }
 			.into_boxed_slice();
 		mem::forget(self);
 		out
@@ -457,7 +458,7 @@ where C: Cursor, T: Bits {
 	where F: FnOnce(&Box<[T]>) -> R {
 		let (data, elts, _, _) = self.bitptr().raw_parts();
 		let b: Box<[T]> = unsafe {
-			Vec::from_raw_parts(data as *mut T, elts, elts)
+			Vec::from_raw_parts(data.w(), elts, elts)
 		}.into_boxed_slice();
 		let out = func(&b);
 		mem::forget(b);
@@ -667,10 +668,14 @@ where C: Cursor, T: 'a + Bits {
 	}
 }
 
-/// `BitBox` is safe to send across threads, but cannot be shared between them.
+/// `BitBox` is safe to move across thread boundaries, as is `&mut BitBox`.
 unsafe impl<C, T> Send for BitBox<C, T>
 where C: Cursor, T: Bits {}
 
+/// `&BitBox` is safe to move across thread boundaries.
+unsafe impl<C, T> Sync for BitBox<C, T>
+where C: Cursor, T: Bits {}
+
 impl<C, T> Add<Self> for BitBox<C, T>
 where C: Cursor, T: Bits {
 	type Output = Self;
