@@ -43,46 +43,40 @@ extern crate alloc;
 #[cfg(feature = "std")]
 extern crate core;
 
+#[cfg(feature = "serde")]
+extern crate serde;
+
+#[cfg(all(test, feature = "serde"))]
+extern crate serde_test;
+
 #[macro_use]
 mod macros;
 
-mod bits;
-mod cursor;
+pub mod bits;
+pub mod cursor;
+mod domain;
 mod pointer;
-mod slice;
+pub mod prelude;
+pub mod slice;
 
-#[cfg(feature = "alloc")]
-mod boxed;
+#[cfg(any(feature = "alloc", feature = "std"))]
+pub mod boxed;
 
-#[cfg(feature = "alloc")]
-mod vec;
+#[cfg(any(feature = "alloc", feature = "std"))]
+pub mod vec;
 
-use crate::{
-	bits::BitIdx,
-	pointer::BitPtr,
-};
+#[cfg(feature = "atomic")]
+mod atomic;
 
-pub use crate::{
-	bits::Bits,
-	cursor::{
-		Cursor,
-		BigEndian,
-		LittleEndian,
-	},
-	slice::BitSlice,
-};
-
-#[cfg(feature = "alloc")]
-pub use crate::{
-	boxed::BitBox,
-	vec::BitVec,
-};
+#[cfg(feature = "serde")]
+mod serdes;
 
 /// Expose crate internals for use in doctests and external tests.
 #[cfg(feature = "testing")]
 pub mod testing {
 	pub use crate::{
 		bits::*,
+		domain::*,
 		macros::*,
 		pointer::*,
 		slice::*,
